;**********************************************************************
;                                                                     *
;    Filename:         MagiDavid AL101 - Lab 005.asm                  *
;    Date:             10/21/2011                                     *
;    File Version:     1.0                                            *
;                                                                     *
;    Author:           David Hoffman                                  *
;    Company:          NeoLoch, LLC                                   *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: P16F882.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    License: Creative Commons                                        *
;    Attribution-NonCommercial-ShareAlike 3.0 Unported License.       *
;             http://creativecommons.org/licenses/by-nc-sa/3.0        *
;                                                                     *
;**********************************************************************

     LIST      p=16F882            ; list directive to define processor
     #INCLUDE <p16F882.inc>        ; processor specific variable definitions

     __CONFIG _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT 
     __CONFIG _CONFIG2, _WRT_OFF & _BOR40V

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.
; Note that the _DEBUG_ON is changed by selecting a debugger or programmer.  
; There isn't much advantage to setting it in the code.  

#DEFINE		DOT_ROW1	PORTA,7
#DEFINE		DOT_ROW2	PORTB,0	
#DEFINE		DOT_ROW3	PORTB,1
#DEFINE		DOT_ROW4	PORTB,2
#DEFINE		DOT_ROW5	PORTB,3
#DEFINE		DMD_PORT	PORTA

DMD_VAR		UDATA
DMD_DISPLAY	RES 5			;RESERVE 5 MEMORY LOCATIONS FOR DISPLAY DATA.

DMD_VAR_SHR	UDATA_SHR
DMD_LOOP	RES 1
DMD_LOOP2	RES 1
DMD_COUNT	RES 1
DMD_TEMP	RES 1

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RESET_VECTOR    CODE   0x0000
          goto    START             ; When using debug header, ICD2 may not stop
                                    ; on instruction 0 during reset.

;------------------------------------------------------------------------------
; CONFIGURATION CODE
;------------------------------------------------------------------------------
;START	ORG	0X05
MAIN_PROG       CODE
START
	BANKSEL PORTA
	CLRF	PORTA

	BANKSEL	ANSEL
	CLRF	ANSEL

	BANKSEL	TRISA
	MOVLW	0X00
	MOVWF	TRISA

	CLRF	DMD_COUNT
	CLRF	DMD_LOOP
	CLRF	DMD_LOOP2

	BANKSEL	ANSELH			;SELECT THE BANK THAT ANSELH IS IN.
	CLRF	ANSELH			;SET PORTB AS DIGITAL I/O.
	BANKSEL	TRISB			;SELECT THE BANK THAT TRISB IS IN.
	CLRW
	MOVWF	TRISB			;SET PORT B AS DIGITAL OUTPUT.
	BANKSEL PORTB			;SELECT BANK THAT PORT B IS IN.
	MOVLW	0XFF			;COPY 0XFF TO W.
	MOVWF	PORTB			;TURN ALL PORTB BITS ON.


;------------------------------------------------------------------------------
; MAIN CODE
;------------------------------------------------------------------------------
MAIN
	BANKSEL	PORTA

	CALL	DMD_CHAR1
	CALL	DMD_UPDATE

	GOTO	MAIN			;LOOP AROUND AND IT ALL OVER AGAIN.



;------------------------------------------------------------------------------
; DOT MATRIX DISPLAY UPDATE
;------------------------------------------------------------------------------
DMD_UPDATE
	BANKSEL	DMD_PORT
	BSF	DOT_ROW5		;TURN OFF ROW 5.
	MOVF	DMD_DISPLAY,W		;NO - COPY DATA TO BE DISPLAYED.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY PORT.
	BCF	DOT_ROW1		;TURN ON ROW 1.
	CALL	DMD_PAUSE		;LEAVE DATA ON PORT FOR A LITTLE WHILE.

	BANKSEL	DMD_PORT
	BSF	DOT_ROW1		;TURN OFF ROW 5.
	MOVF	DMD_DISPLAY+1,W		;NO - COPY DATA TO BE DISPLAYED.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY PORT.
	BCF	DOT_ROW2		;TURN ON ROW 1.
	CALL	DMD_PAUSE		;LEAVE DATA ON PORT FOR A LITTLE WHILE.

	BANKSEL	DMD_PORT
	BSF	DOT_ROW2		;TURN OFF ROW 5.
	MOVF	DMD_DISPLAY+2,W		;NO - COPY DATA TO BE DISPLAYED.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY PORT.
	BCF	DOT_ROW3		;TURN ON ROW 1.
	CALL	DMD_PAUSE		;LEAVE DATA ON PORT FOR A LITTLE WHILE.

	BANKSEL	DMD_PORT
	BSF	DOT_ROW3		;TURN OFF ROW 5.
	MOVF	DMD_DISPLAY+3,W		;NO - COPY DATA TO BE DISPLAYED.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY PORT.
	BCF	DOT_ROW4		;TURN ON ROW 1.
	CALL	DMD_PAUSE		;LEAVE DATA ON PORT FOR A LITTLE WHILE.

	BANKSEL	DMD_PORT
	BSF	DOT_ROW4		;TURN OFF ROW 5.
	MOVF	DMD_DISPLAY+4,W		;NO - COPY DATA TO BE DISPLAYED.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY PORT.
	BCF	DOT_ROW5		;TURN ON ROW 1.
	CALL	DMD_PAUSE		;LEAVE DATA ON PORT FOR A LITTLE WHILE.

	RETURN

DMD_DATA_SHOW
	MOVWF	DMD_TEMP
	BSF	DMD_TEMP,7		;SET BIT 7 IS SET SO WE DON'T TURN ON ROW 1.
	MOVF	DMD_TEMP,W
	BANKSEL	DMD_PORT
	MOVWF	PORTA
	RETURN

;------------------------------------------------------------------------------
; PAUSE AND REFRESH THE LED FOR A SHORT WHILE.
;------------------------------------------------------------------------------
DMD_PAUSE
	DECFSZ	DMD_LOOP,F
	GOTO	DMD_PAUSE
	RETURN

;------------------------------------------------------------------------------
; CHARACTER DATA. THE FOLLOWING ROUTINE CONTAINS HARDCODED INSTRUCTION TO
; GENERATE THE NUMBER '10' ON THE DISPLAY. 
;------------------------------------------------------------------------------
DMD_CHAR1
	MOVLW	B'00100111'
	MOVWF	DMD_DISPLAY+4
	MOVLW	B'01100101'
	MOVWF	DMD_DISPLAY+3
	MOVLW	B'00100101'
	MOVWF	DMD_DISPLAY+2
	MOVLW	B'00100101'
	MOVWF	DMD_DISPLAY+1
	MOVLW	B'01110111'
	MOVWF	DMD_DISPLAY
	RETURN



          END